"""
Tests for clients in enterprise_reporting.
"""
from unittest.mock import Mock, patch
from urllib.parse import urljoin

import responses

from django.conf import settings
from django.test import TestCase

from enterprise_reporting.clients.enterprise import EnterpriseAPIClient


class TestEnterpriseAPIClient(TestCase):
    """
    Test Enterprise API client used to connect to the Enterprise API.
    """

    def setUp(self):
        self.enterprise_customer_uuid = 'test-enterprise-customer-uuid'
        self.reporting_config = {}
        self.api_response = {
            'results': [{
                'enterprise_name': 'Test Enterprise',
                'enterprise_id': 'test-id'
            }]
        }
        self.mocked_get_endpoint = Mock(return_value=self.api_response)
        self.client = EnterpriseAPIClient(  # pylint: disable=attribute-defined-outside-init
            settings.BACKEND_SERVICE_EDX_OAUTH2_KEY,
            settings.BACKEND_SERVICE_EDX_OAUTH2_SECRET,
        )
        self.client.API_BASE_URL = 'http://localhost-test:8000/'
        super().setUp()

    @responses.activate
    @patch('enterprise_reporting.clients.get_oauth_access_token')
    def test_get_all_enterprise_reporting_configs(self, mock_get_oauth_access_token):
        mock_get_oauth_access_token.return_value = {
            'access_token': 'test_access_token',
            'expires_in': 10,
        }
        url = urljoin(self.client.API_BASE_URL + '/', self.client.ENTERPRISE_REPORTING_ENDPOINT)
        responses.add(
            responses.GET,
            url,
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_all_enterprise_reporting_configs()
        assert results['results'] == self.api_response['results']

    @responses.activate
    @patch('enterprise_reporting.clients.get_oauth_access_token')
    def test_get_enterprise_reporting_configs(self, mock_get_oauth_access_token):
        mock_get_oauth_access_token.return_value = {
            'access_token': 'test_access_token',
            'expires_in': 10,
        }
        url = urljoin(self.client.API_BASE_URL + '/', self.client.ENTERPRISE_REPORTING_ENDPOINT)
        responses.add(
            responses.GET,
            url,
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_enterprise_reporting_configs(self.enterprise_customer_uuid)
        assert results['results'] == self.api_response['results']

    @responses.activate
    @patch('enterprise_reporting.clients.get_oauth_access_token')
    def test_get_customer_catalogs(self, mock_get_oauth_access_token):
        mock_get_oauth_access_token.return_value = {
            'access_token': 'test_access_token',
            'expires_in': 10,
        }
        url = urljoin(self.client.API_BASE_URL + '/', self.client.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT)
        responses.add(
            responses.GET,
            url,
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_customer_catalogs(self.enterprise_customer_uuid)
        assert results['results'] == self.api_response['results']

    @responses.activate
    @patch('enterprise_reporting.clients.get_oauth_access_token')
    def test_get_content_metadata(self, mock_get_oauth_access_token):
        mock_get_oauth_access_token.return_value = {
            'access_token': 'test_access_token',
            'expires_in': 10,
        }
        mocked_get_endpoint = Mock(return_value={
            'results': [{
                'uuid': self.enterprise_customer_uuid
            }]
        })
        url = urljoin(self.client.API_BASE_URL + '/', self.client.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT)
        responses.add(
            responses.GET,
            url,
            json=mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        api_response = {
            'results': [{
                'uuid': self.enterprise_customer_uuid,
                'content_type': 'program',
            }]
        }
        mocked_get_endpoint = Mock(return_value=api_response)
        url = urljoin(self.client.API_BASE_URL + '/', self.client.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT + '/' + self.enterprise_customer_uuid)
        responses.add(
            responses.GET,
            url,
            json=mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_content_metadata(self.enterprise_customer_uuid, self.reporting_config)
        assert results == api_response['results']



"""
Tests for views in the `enterprise_data` module.
"""

from unittest import mock
from uuid import uuid4

import ddt
from pytest import mark
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITransactionTestCase

from enterprise_data.api.v1.serializers import EnterpriseOfferSerializer
from enterprise_data.models import EnterpriseLearnerEnrollment, EnterpriseOffer
from enterprise_data.tests.mixins import JWTTestMixin
from enterprise_data.tests.test_utils import (
    EnterpriseLearnerEnrollmentFactory,
    EnterpriseLearnerFactory,
    EnterpriseOfferFactory,
    UserFactory,
    get_dummy_enterprise_api_data,
)
from enterprise_data_roles.constants import ENTERPRISE_DATA_ADMIN_ROLE
from enterprise_data_roles.models import EnterpriseDataFeatureRole, EnterpriseDataRoleAssignment


@ddt.ddt
@mark.django_db
class TestEnterpriseLearnerEnrollmentViewSet(JWTTestMixin, APITransactionTestCase):
    """
    Tests for EnterpriseLearnerEnrollmentViewSet.
    """

    def setUp(self):
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(name=ENTERPRISE_DATA_ADMIN_ROLE)
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role,
            user=self.user
        )
        self.client.force_authenticate(user=self.user)

        mocked_get_enterprise_customer = mock.patch(
            'enterprise_data.filters.EnterpriseApiClient.get_enterprise_customer',
            return_value=get_dummy_enterprise_api_data()
        )

        self.mocked_get_enterprise_customer = mocked_get_enterprise_customer.start()
        self.addCleanup(mocked_get_enterprise_customer.stop)
        self.enterprise_id = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()

    def tearDown(self):
        super().tearDown()
        EnterpriseLearnerEnrollment.objects.all().delete()

    def test_filter_by_offer_id(self):
        enterprise_learner = EnterpriseLearnerFactory(
            enterprise_customer_uuid=self.enterprise_id
        )

        offer_1_id = 1
        offer_2_id = 2

        learner_enrollment_1 = EnterpriseLearnerEnrollmentFactory(
            offer_id=offer_1_id,
            enterprise_customer_uuid=self.enterprise_id,
            is_consent_granted=True,
            enterprise_user_id=enterprise_learner.enterprise_user_id
        )
        EnterpriseLearnerEnrollmentFactory(
            offer_id=offer_2_id,
            enterprise_customer_uuid=self.enterprise_id,
            is_consent_granted=True,
            enterprise_user_id=enterprise_learner.enterprise_user_id
        )

        url = reverse('v1:enterprise-learner-enrollment-list', kwargs={'enterprise_id': self.enterprise_id})
        response = self.client.get(url, data={'offer_id': offer_1_id})
        results = response.json()['results']
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0]['enrollment_id'], learner_enrollment_1.enrollment_id)

    def test_filter_by_ignore_null_course_list_price(self):
        enterprise_learner = EnterpriseLearnerFactory(
            enterprise_customer_uuid=self.enterprise_id
        )

        EnterpriseLearnerEnrollmentFactory(
            course_list_price=None,
            enterprise_customer_uuid=self.enterprise_id,
            is_consent_granted=True,
            enterprise_user_id=enterprise_learner.enterprise_user_id
        )
        learner_enrollment_with_price = EnterpriseLearnerEnrollmentFactory(
            enterprise_customer_uuid=self.enterprise_id,
            is_consent_granted=True,
            enterprise_user_id=enterprise_learner.enterprise_user_id
        )

        url = reverse('v1:enterprise-learner-enrollment-list', kwargs={'enterprise_id': self.enterprise_id})
        response = self.client.get(url, data={'ignore_null_course_list_price': True})
        results = response.json()['results']
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0]['enrollment_id'], learner_enrollment_with_price.enrollment_id)


@ddt.ddt
@mark.django_db
class TestEnterpriseOffersViewSet(JWTTestMixin, APITransactionTestCase):
    """
    Tests for EnterpriseOfferViewSet.
    """

    def setUp(self):
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(name=ENTERPRISE_DATA_ADMIN_ROLE)
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role,
            user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_customer_uuid_1 = uuid4()
        self.enterprise_offer_1 = EnterpriseOfferFactory(
            enterprise_customer_uuid=self.enterprise_customer_uuid_1
        )

        self.enterprise_customer_2_uuid = uuid4()
        self.enterprise_offer_2 = EnterpriseOfferFactory(
            enterprise_customer_uuid=self.enterprise_customer_2_uuid
        )

        self.set_jwt_cookie()

    def tearDown(self):
        super().tearDown()
        EnterpriseOffer.objects.all().delete()

    def test_list_offers(self):
        enterprise_id = self.enterprise_offer_1.enterprise_customer_uuid
        url = reverse(
            'v1:enterprise-offers-list',
            kwargs={'enterprise_id': enterprise_id}
        )

        response = self.client.get(url)
        assert response.status_code == status.HTTP_200_OK

        expected_results = [EnterpriseOfferSerializer(self.enterprise_offer_1).data]
        response_json = response.json()
        results = response_json['results']
        assert results == expected_results



"""
Tests for clients in enterprise_data.
"""
from unittest.mock import Mock, patch
from urllib.parse import urljoin

import responses
from requests.exceptions import HTTPError
from rest_framework.exceptions import NotFound, ParseError

from django.conf import settings
from django.test import TestCase

from enterprise_data.clients import EnterpriseApiClient
from enterprise_data.tests.test_utils import UserFactory, get_dummy_enterprise_api_data


class TestEnterpriseApiClient(TestCase):
    """
    Test Enterprise API client used to connect to LMS enterprise endpoints
    """

    def setUp(self):
        self.user = UserFactory()
        self.enterprise_id = '0395b02f-6b29-42ed-9a41-45f3dff8349c'
        self.api_response = {
            'count': 1,
            'results': [{
                'enterprise_name': 'Test Enterprise',
                'enterprise_id': 'test-id'
            }]
        }
        self.mocked_get_endpoint = Mock(return_value=self.api_response)
        super().setUp()

    def mock_client(self):
        """
        Set up a mocked Enterprise API Client. Avoiding doing this in setup so we can test __init__.
        """
        self.client = EnterpriseApiClient(  # pylint: disable=attribute-defined-outside-init
            settings.BACKEND_SERVICE_EDX_OAUTH2_PROVIDER_URL,
            settings.BACKEND_SERVICE_EDX_OAUTH2_KEY,
            settings.BACKEND_SERVICE_EDX_OAUTH2_SECRET,
        )

        responses.add(
            responses.POST,
            urljoin(settings.BACKEND_SERVICE_EDX_OAUTH2_PROVIDER_URL + '/', 'access_token'),
            status=200,
            json={
                'access_token': 'test_access_token',
                'expires_in': 10,
            },
            content_type='application/json'
        )

    @responses.activate
    def test_get_enterprise_learner_returns_results_for_user(self):
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/enterprise-learner'),
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_enterprise_learner(self.user)
        assert results == self.api_response['results'][0]

    @responses.activate
    def test_get_enterprise_learner_raises_exception_on_error(self):
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/enterprise-learner'),
            json=self.mocked_get_endpoint(),
            status=404,
            content_type='application/json'
        )
        with self.assertRaises(HTTPError):
            _ = self.client.get_enterprise_learner(self.user)

    @responses.activate
    def test_get_enterprise_learner_returns_none_on_empty_results(self):
        self.mocked_get_endpoint = Mock(return_value={
            'count': 0,
            'results': []
        })
        self.mock_client()

        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/enterprise-learner'),
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_enterprise_learner(self.user)
        self.assertIsNone(results)

    @responses.activate
    def test_get_enterprise_learner_raises_not_found_on_no_results(self):
        self.mocked_get_endpoint = Mock(return_value={})
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/enterprise-learner'),
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        with self.assertRaises(NotFound):
            _ = self.client.get_enterprise_learner(self.user)

    @responses.activate
    def test_get_enterprise_learner_raises_parse_error_on_multiple_results(self):
        self.mocked_get_endpoint = Mock(return_value={
            'count': 2,
            'results': [{}, {}]
        })
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/enterprise-learner'),
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        with self.assertRaises(ParseError):
            _ = self.client.get_enterprise_learner(self.user)

    @responses.activate
    def test_get_enterprise_customer_returns_results_for_user(self):
        self.mocked_get_endpoint = Mock(return_value=get_dummy_enterprise_api_data())
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', f'enterprise/api/v1/enterprise-customer/{self.enterprise_id}'),
            json=self.mocked_get_endpoint(),
            status=200,
            content_type='application/json'
        )
        results = self.client.get_enterprise_customer(self.user, self.enterprise_id)
        assert results == self.mocked_get_endpoint()

    @responses.activate
    def test_get_enterprise_customer_raises_exception_on_error(self):
        self.mocked_get_endpoint = Mock(return_value=get_dummy_enterprise_api_data())
        self.mock_client()
        responses.add(
            responses.GET,
            urljoin(settings.LMS_BASE_URL + '/', f'enterprise/api/v1/enterprise-customer/{self.enterprise_id}'),
            json=self.mocked_get_endpoint(),
            status=404,
            content_type='application/json'
        )
        with self.assertRaises(HTTPError):
            _ = self.client.get_enterprise_customer(self.user, self.enterprise_id)

    @responses.activate
    @patch('enterprise_data.clients.TieredCache')
    def test_get_enterprise_customer_returns_results_from_cashe(self, tired_cache_mock):
        mocked_value = get_dummy_enterprise_api_data()
        self.mock_client()
        tired_cache_mock.get_cached_response.return_value = Mock(
            value=mocked_value,
            is_found=True
        )
        results = self.client.get_enterprise_customer(self.user, self.enterprise_id)
        tired_cache_mock.get_cached_response.assert_called_once()
        assert results == mocked_value



# Generated by Django 3.2.13 on 2022-06-22 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0028_enterpriselearnerenrollment_offer_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseOffer',
            fields=[
                ('offer_id', models.PositiveIntegerField(primary_key=True, serialize=False)),
                ('enterprise_customer_uuid', models.UUIDField()),
                ('enterprise_name', models.CharField(max_length=255, null=True)),
                ('sum_amount_learner_paid', models.FloatField(null=True)),
                ('sum_course_price', models.FloatField(null=True)),
                ('status', models.CharField(max_length=255, null=True)),
                ('offer_type', models.CharField(max_length=255, null=True)),
                ('date_created', models.DateTimeField(null=True)),
                ('emails_for_usage_alert', models.CharField(max_length=1023, null=True)),
                ('discount_type', models.CharField(max_length=64, null=True)),
                ('discount_value', models.FloatField(null=True)),
                ('max_discount', models.FloatField(null=True)),
                ('total_discount_ecommerce', models.FloatField(null=True)),
                ('amount_of_offer_spent', models.FloatField(null=True)),
                ('percent_of_offer_spent', models.FloatField(null=True)),
                ('remaining_balance', models.FloatField(null=True)),
            ],
            options={
                'verbose_name': 'Enterprise Offer',
                'verbose_name_plural': 'Enterprise Offers',
                'db_table': 'enterprise_offer_aggregates',
            },
        ),
    ]



# Generated by Django 1.11.5 on 2018-06-14 01:08


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0007_auto_20180612_0534'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='coupon_code',
            field=models.CharField(max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='coupon_name',
            field=models.CharField(max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='course_price',
            field=models.DecimalField(decimal_places=2, max_digits=12, null=True),
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='discount_price',
            field=models.DecimalField(decimal_places=2, max_digits=12, null=True),
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='final_grade',
            field=models.FloatField(null=True),
        ),
    ]



"""
Tests for the `edx-enterprise` api v1 serializer module.
"""

import ddt
from pytest import mark
from rest_framework.test import APITransactionTestCase

from enterprise_data.api.v1.serializers import EnterpriseLearnerEnrollmentSerializer
from enterprise_data.tests.test_utils import EnterpriseLearnerEnrollmentFactory, EnterpriseLearnerFactory


@mark.django_db
@ddt.ddt
class TestEnterpriseLearnerEnrollmentSerializer(APITransactionTestCase):
    """
    Tests for `enterprise_learner_enrollment` API serializer.
    """

    def setUp(self):
        super().setUp()
        ent_user = EnterpriseLearnerFactory()
        self.enrollment = EnterpriseLearnerEnrollmentFactory(
            enterprise_user_id=ent_user.enterprise_user_id,
            is_consent_granted=True
        )

    @ddt.data(
        (0.0, 0.0),
        (3600, 1.0),
    )
    @ddt.unpack
    def test_total_learning_time_hours(self, total_learning_time_seconds, expected_total_learning_time_seconds):
        self.enrollment.total_learning_time_seconds = total_learning_time_seconds
        self.enrollment.save()
        serializer = EnterpriseLearnerEnrollmentSerializer(self.enrollment)
        assert serializer.data['total_learning_time_hours'] == expected_total_learning_time_seconds



"""
management command for creating dummy enterprise offers.
"""


import sys

from django.core.management.base import BaseCommand, CommandError

import enterprise_data.tests.test_utils


class Command(BaseCommand):
    """ management command class """
    help = 'Creates an EnterpriseOffer with randomized attributes'

    def add_arguments(self, parser):
        parser.add_argument(
            'enterprise_customer_uuid',
            type=str,
            help='UUID for an enterprise'
        )

    def handle(self, *args, **options):
        enterprise_customer_uuid = options['enterprise_customer_uuid']

        try:
            enterprise_data.tests.test_utils.EnterpriseOfferFactory(
                enterprise_customer_uuid=enterprise_customer_uuid,
            )
            info = (
                f'Created EnterpriseOffer for enterprise {enterprise_customer_uuid}\n'
            )
            sys.stdout.write(info)
        except Exception as exc:
            info = (
                f'Error trying to create EnterpriseOffer for enterprise {enterprise_customer_uuid}\n'
            )
            raise CommandError(info) from exc



"""
Client for connecting to the LMS Enterprise endpoints.
"""


import logging
import os
from collections import OrderedDict
from urllib.parse import urljoin

from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin

from enterprise_reporting import utils
from enterprise_reporting.clients import EdxOAuth2APIClient

LOGGER = logging.getLogger(__name__)


class EnterpriseCatalogAPIClient(EdxOAuth2APIClient):
    """
    Client for connect to the Enterprise Catalog Service API.
    """
    API_BASE_URL = EdxOAuth2APIClient.ENTERPRISE_CATALOG_ROOT_URL + '/api/v1/'

    APPEND_SLASH = True
    ENTERPRISE_CATALOGS_ENDPOINT = 'enterprise-catalogs'
    GET_CONTENT_METADATA_ENDPOINT = ENTERPRISE_CATALOGS_ENDPOINT + '/{}/get_content_metadata'

    ENTERPRISE_REPORTING_ENDPOINT = 'enterprise_customer_reporting'
    ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT = 'enterprise_catalogs'

    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    def transform_get_content_metadata(self, traversed_metadata):
        """
        Helper method to transform a response (already traversed pagination) from the enterprise-catalog service's `get_content_metadata`.
        endpoint.

        Note: as of June 2022 the data source for content metadata changed from edx platform to the enterprise catalog
        service. As such, content metadata coming from the enterprise catalog service needs to be formatted to match
        the old endpoints data structure.
        """
        content_metadata = OrderedDict()
        for item in traversed_metadata:
            content_id = utils.get_content_metadata_item_id(item)

            # Check if the item is a courserun
            if item.get('content_type') == 'courserun':
                # Courserun content metadata needs no massaging as it's the same in platform as it is in the
                # enterprise catalog service
                formatted_metadata = item
            else:
                # Course content metadata differs slightly between platform and the catalog service, so we need to
                # massage the response ie filter out new, not expected fields
                item_crs = item.get('course_runs', [])
                formatted_course_runs = []
                for cr in item_crs:
                    # While courseruns as a content metadata item did not change between old and new api endpoints,
                    # the courserun attribute of a course did, so we need to do some formatting.
                    formatted_course_run = {
                        'key': cr.get('key'),
                        'enrollment_start': cr.get('enrollment_start'),
                        'enrollment_end': cr.get('enrollment_end'),
                        'go_live_date': cr.get('go_live_date'),
                        'start': cr.get('start'),
                        'end': cr.get('end'),
                        'modified': cr.get('modified'),
                        'availability': cr.get('availability'),
                        'status': cr.get('status'),
                        'pacing_type': cr.get('pacing_type'),
                        # New endpoint uses `type` instead of enrollment mode
                        'enrollment_mode': cr.get('type'),
                        'min_effort': cr.get('min_effort'),
                        'max_effort': cr.get('max_effort'),
                        'weeks_to_complete': cr.get('weeks_to_complete'),
                        'estimated_hours': cr.get('estimated_hours'),
                        'first_enrollable_paid_seat_price': cr.get('first_enrollable_paid_seat_price'),
                        'is_enrollable': cr.get('is_enrollable'),
                    }
                    formatted_course_runs.append(formatted_course_run)

                # Subject attributes also need reformatting
                subjects = item.get('subjects', [])
                formatted_subjects = []
                for subject in subjects:
                    subject_name = subject.get('name')
                    if subject_name:
                        formatted_subjects.append(subject_name)

                formatted_metadata = {
                    'active': item.get('active'),
                    'aggregation_key': item.get('aggregation_key'),
                    'card_image_url': item.get('card_image_url'),
                    'content_type': item.get('content_type'),
                    'course_ends': item.get('course_ends'),
                    'course_runs': formatted_course_runs,
                    'end_date': item.get('end_date'),
                    'enrollment_url': item.get('enrollment_url'),
                    'full_description': item.get('full_description'),
                    'image_url': item.get('image_url'),
                    'key': item.get('key'),
                    'languages': item.get('languages'),
                    'organizations': item.get('organizations'),
                    'seat_types': item.get('seat_types'),
                    'short_description': item.get('short_description'),
                    'skill_names': item.get('skill_names'),
                    'skills': item.get('skills'),
                    'subjects': formatted_subjects,
                    'title': item.get('title'),
                    'uuid': item.get('uuid'),
                }

            content_metadata[content_id] = formatted_metadata

        return content_metadata

    @EdxOAuth2APIClient.refresh_token
    def get_content_metadata(self, enterprise_customer_catalogs):
        """Return all content metadata contained in the catalogs associated with a reporting config."""
        content_metadata = OrderedDict()
        for catalog in enterprise_customer_catalogs.get('results', []):
            traversed_metadata = self._load_data(
                self.GET_CONTENT_METADATA_ENDPOINT.format(catalog['uuid']),
                should_traverse_pagination=True,
                querystring={'page_size': self.PAGE_SIZE},
            )
            transformed_metadata = self.transform_get_content_metadata(traversed_metadata.get('results'))
            content_metadata.update(transformed_metadata)

        # We only made this a dictionary to help filter out duplicates by a common key. We just want values now.
        return list(content_metadata.values())


class EnterpriseAPIClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Enterprise API.
    """
    API_BASE_URL = urljoin(EdxOAuth2APIClient.LMS_ROOT_URL + '/', 'enterprise/api/v1/')

    ENTERPRISE_REPORTING_ENDPOINT = 'enterprise_customer_reporting'
    ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT = 'enterprise_catalogs'

    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_all_enterprise_reporting_configs(self, **kwargs):
        """
        Query the enterprise customer reporting endpoint for all available configs.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            should_traverse_pagination=True,
            **kwargs
        )

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_reporting_configs(self, enterprise_customer_uuid, **kwargs):
        """
        Query the enterprise customer reporting endpoint for a given enterprise customer.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            querystring={'enterprise_customer': enterprise_customer_uuid},
            should_traverse_pagination=True,
            **kwargs
        )

    @EdxOAuth2APIClient.refresh_token
    def get_content_metadata(self, enterprise_customer_uuid, reporting_config):
        """Return all content metadata contained in the catalogs associated with an Enterprise Customer."""
        content_metadata = OrderedDict()

        enterprise_customer_catalogs = utils.extract_catalog_uuids_from_reporting_config(reporting_config)
        if not enterprise_customer_catalogs.get('results'):
            enterprise_customer_catalogs = self._load_data(
                self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                should_traverse_pagination=True,
                querystring={
                    'enterprise_customer': enterprise_customer_uuid,
                    'page_size': self.PAGE_SIZE,
                },
            )
        for catalog in enterprise_customer_catalogs.get('results', []):
            catalog_content = self._load_data(
                self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                resource_id=catalog['uuid'],
                should_traverse_pagination=True,
                querystring={'page_size': self.PAGE_SIZE},
            )
            # It's possible that there are duplicate items.
            # Filter them out by assigning common items to their common identifier in a dictionary.
            for item in catalog_content['results']:
                key = 'uuid' if item['content_type'] == 'program' else 'key'
                content_metadata[item[key]] = item

        # We only made this a dictionary to help filter out duplicates by a common key. We just want values now.
        return list(content_metadata.values())

    @EdxOAuth2APIClient.refresh_token
    def get_customer_catalogs(self, enterprise_customer_uuid):
        """Return all catalog uuids owned by an Enterprise Customer."""
        return self._load_data(
            self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
            should_traverse_pagination=True,
            querystring={
                'enterprise_customer': enterprise_customer_uuid,
                'page_size': self.PAGE_SIZE,
            },
        )


class EnterpriseDataApiClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Enterprise Data API.
    """

    API_BASE_URL = urljoin(os.getenv('ANALYTICS_API_URL', default='') + '/', 'enterprise/api/v0')
    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_enrollments(self, enterprise_customer_uuid):
        return self._load_data(
            'enterprise',
            resource_id=enterprise_customer_uuid,
            detail_resource='enrollments',
            should_traverse_pagination=True,
            querystring={'page_size': self.PAGE_SIZE},
        )


class EnterpriseDataV1ApiClient(EnterpriseDataApiClient):
    """
    Client for connecting to the Enterprise Data V1 API.
    """

    API_BASE_URL = urljoin(os.getenv('ANALYTICS_API_URL', default='') + '/', 'enterprise/api/v1')


class AnalyticsDataApiClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Analytics Data API.
    """

    API_BASE_URL = urljoin(os.getenv('ANALYTICS_API_URL', default='') + '/', 'api/v0')
    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_engagements(self, enterprise_customer_uuid):
        return self._load_data(
            'enterprise',
            resource_id=enterprise_customer_uuid,
            detail_resource='engagements',
            should_traverse_pagination=True,
            querystring={'page_size': self.PAGE_SIZE},
        )



#!/usr/bin/env python



import os
import re
import sys

from setuptools import setup


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

VERSION = get_version("enterprise_data", "__init__.py")

if sys.argv[-1] == "tag":
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (VERSION, VERSION))
    os.system("git push --tags")
    sys.exit()


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns a list of requirement strings.
    """
    requirements = set()
    for path in requirements_paths:
        with open(path) as reqs:
            requirements.update(
                line.split('#')[0].strip() for line in reqs
                if is_requirement(line.strip())
            )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement;
    that is, it is not blank, a comment, a URL, or an included file.
    """
    return line and not line.startswith(('-r', '#', '-e', 'git+', '-c'))


setup(
    name="edx-enterprise-data",
    version=VERSION,
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    description="""Enterprise Reporting""",
    long_description="Tools and products related to providing access to Enterprise data.",
    author="edX",
    author_email="oscm@edx.org",
    url="https://github.com/edx/edx-enterprise-data",
    packages=[
        'enterprise_data',
        'enterprise_reporting',
        'enterprise_data_roles',
    ],
    include_package_data=True,
    install_requires=load_requirements('requirements/base.in'),
    extras_require={'reporting':load_requirements('requirements/reporting.in')},
    license="AGPL 3.0",
    zip_safe=False,
)



"""
Clients used to connect to other systems.
"""


import logging
from urllib.parse import urljoin

from edx_django_utils.cache import TieredCache
from edx_rest_api_client.client import OAuthAPIClient
from requests.exceptions import HTTPError, RequestException
from rest_framework.exceptions import NotFound, ParseError

from django.conf import settings

from enterprise_data.utils import get_cache_key, update_session_with_enterprise_data

DEFAULT_REPORTING_CACHE_TIMEOUT = 60 * 60 * 6  # 6 hours (Value is in seconds)
LOGGER = logging.getLogger('enterprise_data')


class EnterpriseApiClient(OAuthAPIClient):
    """
    The EnterpriseApiClient is used to communicate with the enterprise API endpoints in the LMS.

    This class is a sub-class of the OAuthAPIClient
    (https://github.com/openedx/edx-rest-api-client).
    """

    API_BASE_URL = urljoin(settings.LMS_BASE_URL + '/', 'enterprise/api/v1/')

    def get_enterprise_learner(self, user):
        """
        Get an enterprise learner record for a given user with the enterprise association.

        Returns: learner record or None if unable to retrieve or no Enterprise Learner exists
        """
        try:
            querystring = {'username': user.username}
            url = urljoin(self.API_BASE_URL, 'enterprise-learner')
            response = self.get(url, params=querystring)
            response.raise_for_status()
            data = response.json()
        except (HTTPError, RequestException) as exc:
            LOGGER.warning(
                "[Data Overview Failure] Unable to retrieve Enterprise Customer Learner details. "
                f"User: {user.username}, Exception: {exc}"
            )
            raise

        if data.get('results', None) is None:
            LOGGER.warning(
                f"[Data Overview Failure] Enterprise Customer Learner details could not be found. User: {user.username}"
            )
            raise NotFound('Unable to process Enterprise Customer Learner details for user {}: No Results Found'
                           .format(user.username))

        if data['count'] > 1:
            LOGGER.warning(
                f"[Data Overview Failure] Multiple Enterprise Customer Learners found. User: {user.username}"
            )
            raise ParseError(f'Multiple Enterprise Customer Learners found for user {user.username}')

        if data['count'] == 0:
            return None

        return data['results'][0]

    def get_enterprise_customer(self, user, enterprise_id):
        """
        Get the enterprises that this user has access to.
        """
        LOGGER.info(f'[EnterpriseApiClient] getting latest info for enterprise customer:{enterprise_id}')
        cache_key = get_cache_key(
            resource='enterprise-customer',
            user=user.username,
            enterprise_customer=enterprise_id,
        )
        cached_response = TieredCache.get_cached_response(cache_key)
        if cached_response.is_found:
            LOGGER.info(
                f'[EnterpriseApiClient] cache info found for enterprise customer:{enterprise_id}'
                f' with {cached_response.value}'
            )
            return cached_response.value

        LOGGER.info(f'[EnterpriseApiClient] No cached info found for enterprise customer:{enterprise_id}')
        url = urljoin(self.API_BASE_URL, f'enterprise-customer/{enterprise_id}')

        try:
            response = self.get(url)
            response.raise_for_status()
            data = response.json()
        except (HTTPError, RequestException) as exc:
            LOGGER.warning(
                "[Data Overview Failure] Unable to retrieve Enterprise Customer details. "
                f"User: {user.username}, Enterprise: {enterprise_id}, Exception: {exc}"
            )
            raise

        LOGGER.info(f'[EnterpriseApiClient] Setting cached for enterprise customer:{enterprise_id} with {data}')
        TieredCache.set_all_tiers(cache_key, data, DEFAULT_REPORTING_CACHE_TIMEOUT)

        return data

    def get_enterprise_and_update_session(self, request):
        """
        Get the enterprise customer data and updates the session.

        Returns:
            Enterprise Customer or None if unable to get enterprise information
        """
        enterprise_id = request.parser_context.get('kwargs', {}).get('enterprise_id', '')
        session = request.session
        enterprise_data = None
        LOGGER.info(f'[EnterpriseApiClient] Start updating session for enterprise customer: {enterprise_id}')

        audit_enrollment_check = enterprise_id not in session.get('enable_audit_data_reporting', {})
        data_sharing_consent_check = enterprise_id not in session.get('enforce_data_sharing_consent', {})

        if audit_enrollment_check or data_sharing_consent_check:
            LOGGER.info(f'[EnterpriseApiClient] getting latest info for enterprise customer: {enterprise_id}')
            enterprise_data = self.get_enterprise_customer(request.user, enterprise_id)
            enable_audit_data_reporting = False
            enforce_data_sharing_consent = False

            if enterprise_data:
                enable_audit_data_reporting = enterprise_data.get('enable_audit_data_reporting', False)
                enforce_data_sharing_consent = enterprise_data.get('enforce_data_sharing_consent', '')

            updated_values = {
                'enable_audit_data_reporting': enable_audit_data_reporting,
                'enforce_data_sharing_consent': enforce_data_sharing_consent,
            }
            LOGGER.info(f'[EnterpriseApiClient] updating session for enterprise: {enterprise_id} with {updated_values}')
            update_session_with_enterprise_data(request, enterprise_id, **updated_values)

        return enterprise_data
